//---------------------------------------------------------------------------

#include <fmx.h>
#pragma hdrstop

#include "MainFormU.h"
#include <Androidapi.Helpers.hpp>
#include <Androidapi.Jni.JavaTypes.hpp>
#include <Androidapi.Jni.Os.hpp>
#include <Androidapi.Jni.Widget.hpp>
#include <Androidapi.JniBridge.hpp>
#include <FMX.Helpers.Android.hpp>
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.fmx"
TMainForm *MainForm;
//---------------------------------------------------------------------------
__fastcall TMainForm::TMainForm(TComponent* Owner)
    : TForm(Owner)
{
    TMessageManager::DefaultManager->SubscribeToMessage(
        __classid(TPermissionsRequestResultMessage), HandlePermissionsRequest);

}
//---------------------------------------------------------------------------
__fastcall TMainForm::~TMainForm()
{
    TMessageManager::DefaultManager->Unsubscribe(
        __classid(TPermissionsRequestResultMessage), HandlePermissionsRequest);
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::Button1Click(TObject *Sender)
{
    // API 23+ calls guarded by a runtime check against the OS SDK version
    if (TJBuild_VERSION::JavaClass->SDK_INT >= 23)
    {
        if ((TAndroidHelper::Activity->checkSelfPermission(TJManifest_permission::JavaClass->ACCESS_COARSE_LOCATION) !=
             TJPackageManager::JavaClass->PERMISSION_GRANTED) &&
            (TAndroidHelper::Activity->checkSelfPermission(TJManifest_permission::JavaClass->ACCESS_FINE_LOCATION) !=
             TJPackageManager::JavaClass->PERMISSION_GRANTED))
            RequestPermissions();
        else // Permission has already been granted
            SetupMap(true);
    }
    else
        SetupMap(true);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Toast(String Msg, int Duration)
{
    CallInUIThread(
        [Msg, Duration]()
        {
            TJToast::JavaClass->makeText(
                TAndroidHelper::Context,
                StrToJCharSequence(Msg),
                Duration)->show();
        });
}
//---------------------------------------------------------------------------
void TMainForm::RequestPermissions()
{
    TJavaObjectArray__1<_di_JString> *Perms = new TJavaObjectArray__1<_di_JString>(2);
    Perms->Items[0] = TJManifest_permission::JavaClass->ACCESS_COARSE_LOCATION;
    Perms->Items[1] = TJManifest_permission::JavaClass->ACCESS_FINE_LOCATION;
    // Since this whole procedure is only called in an API-guarded manner (from Button1Click),
    // there is no need to check the OS version here
    TAndroidHelper::Activity->requestPermissions(Perms, MY_PERMISSIONS_REQUEST_ACCESS_LOCATION);
    // MY_PERMISSIONS_REQUEST_ACCESS_LOCATION is an app-defined int constant.
    // The callback method, OnPermissionsRequest, gets the result of the request.
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::HandlePermissionsRequest(TObject *Sender, TMessageBase *M)
{
    TPermissionsRequestResultMessage *Msg = dynamic_cast<TPermissionsRequestResultMessage *>(M);
    if (Msg)
    {
        TPermissionsRequestResultData MessageData = Msg->Value;
        OnPermissionsRequest(MessageData.RequestCode, MessageData.Permissions, MessageData.GrantResults);
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::OnPermissionsRequest(int ARequestCode,
    TJavaObjectArray__1<_di_JString> *APermissions, TJavaArray__1<int> *AGrantResults)
{
    if (ARequestCode == MY_PERMISSIONS_REQUEST_ACCESS_LOCATION)
    {
        bool LocationPermissionGranted =
            (AGrantResults->Length == 2) &&
            (AGrantResults->Items[0] == TJPackageManager::JavaClass->PERMISSION_GRANTED) &&
            (AGrantResults->Items[1] == TJPackageManager::JavaClass->PERMISSION_GRANTED);
        if (LocationPermissionGranted)
            Toast("User granted permission", TJToast::JavaClass->LENGTH_SHORT);
        else
            Toast("User denied permission", TJToast::JavaClass->LENGTH_SHORT);

        SetupMap(LocationPermissionGranted);
    }
}
//---------------------------------------------------------------------------
void TMainForm::SetupMap(bool ALocationPermissionGranted)
{
    // Remember to set up the API key in the Version Info page of the Projct options dialog, as per:
    // http://docwiki.embarcadero.com/RADStudio/en/Configuring_Android_Applications_to_Use_Google_Maps#Configuring_Specific_Project_Options
    if (FMapView == NULL)
    {
        FMapView = new TMapView(this);
        FMapView->Align = TAlignLayout::Client;
        FMapView->LayerOptions = TMapLayerOptions() << TMapLayerOption::PointsOfInterest << TMapLayerOption::Buildings << TMapLayerOption::Traffic;
        FMapView->ControlOptions = TMapControlOptions() << TMapControlOption::Zoom << TMapControlOption::Compass;
        FMapView->Parent = MapPanel;
    }
    if (ALocationPermissionGranted)
    {
        // These options only get respected/actioned if the app has one or both access location permissions
        FMapView->LayerOptions = FMapView->LayerOptions + TMapLayerOptions() << TMapLayerOption::UserLocation;
        FMapView->ControlOptions = FMapView->ControlOptions + TMapControlOptions() << TMapControlOption::MyLocation;
    }
}
//---------------------------------------------------------------------------

