//---------------------------------------------------------------------------

#include <fmx.h>
#ifdef __ANDROID__
    #include <Androidapi.Helpers.hpp>
    #include <Androidapi.JNI.Os.hpp>
    #include <Androidapi.Jni.Widget.hpp>
    #include <FMX.Helpers.Android.hpp>
#endif
#pragma hdrstop

#include "MainFormU.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.fmx"
TMainForm *MainForm;
//---------------------------------------------------------------------------
__fastcall TMainForm::TMainForm(TComponent* Owner)
    : TForm(Owner)
{
#ifdef __ANDROID__
    // Note we can alternatively use literal strings:
    //   'android.permission.ACCESS_COARSE_LOCATION'
    //   'android.permission.ACCESS_FINE_LOCATION'
    // according to the documentation:
    //   https://developer.android.com/reference/android/Manifest.permission#ACCESS_COARSE_LOCATION
    //   https://developer.android.com/reference/android/Manifest.permission#ACCESS_FINE_LOCATION
    FPermissionCoarseLocation = JStringToString(TJManifest_permission::JavaClass->ACCESS_COARSE_LOCATION);
    FPermissionFineLocation = JStringToString(TJManifest_permission::JavaClass->ACCESS_FINE_LOCATION);
#endif
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Button1Click(TObject *Sender)
{
    DynamicArray<String> permissions;
    permissions.Length = 2;
    permissions[0] = FPermissionCoarseLocation;
    permissions[1] = FPermissionFineLocation;

    PermissionsService()->RequestPermissions(permissions, LocationPermissionRequestResult);
}
//---------------------------------------------------------------------------
#ifdef __ANDROID__
void __fastcall TMainForm::Toast(String Msg, int Duration)
{
    CallInUIThread(
        [Msg, Duration]()
        {
            TJToast::JavaClass->makeText(
                TAndroidHelper::Context,
                StrToJCharSequence(Msg),
                Duration)->show();
        });
}
#endif

void __fastcall TMainForm::LocationPermissionRequestResult(TObject *Sender, const DynamicArray<String> APermissions, const DynamicArray<TPermissionStatus> AGrantResults)
{
    // 2 permissions involved: ACCESS_COARSE_LOCATION and ACCESS_FINE_LOCATION
    bool LocationPermissionGranted = (AGrantResults.Length == 2) &&
         (AGrantResults[0] == TPermissionStatus::Granted) &&
         (AGrantResults[1] == TPermissionStatus::Granted);
    if (LocationPermissionGranted)
        Toast("User granted permission", TJToast::JavaClass->LENGTH_SHORT);
    else
        Toast("User denied permission", TJToast::JavaClass->LENGTH_SHORT);
    SetupMap(LocationPermissionGranted);
}

void __fastcall TMainForm::SetupMap(bool ALocationPermissionGranted)
{
    // Remember to set up the API key in the Version Info page of the Projct options dialog, as per:
    // http://docwiki.embarcadero.com/RADStudio/en/Configuring_Android_Applications_to_Use_Google_Maps#Configuring_Specific_Project_Options
    if (FMapView == NULL)
    {
        FMapView = new TMapView(this);
        FMapView->Align = TAlignLayout::Client;
        FMapView->LayerOptions = TMapLayerOptions() << TMapLayerOption::PointsOfInterest << TMapLayerOption::Buildings << TMapLayerOption::Traffic;
        FMapView->ControlOptions = TMapControlOptions() << TMapControlOption::Zoom << TMapControlOption::Compass;
        FMapView->Parent = MapPanel;
    }
    if (ALocationPermissionGranted)
    {
        // These options only get respected/actioned if the app has one or both access location permissions
        FMapView->LayerOptions = FMapView->LayerOptions + TMapLayerOptions() << TMapLayerOption::UserLocation;
        FMapView->ControlOptions = FMapView->ControlOptions + TMapControlOptions() << TMapControlOption::MyLocation;
    }
}
